@@ -570,7 +570,7 @@ class Threading:
 
         return unconstrained
 
-    async def check_days_constraint(self, args, name):
+    async def check_days_constraint(self, args):
         unconstrained = True
         if "constrain_days" in args:
             days = args["constrain_days"]
@@ -584,6 +584,13 @@ class Threading:
 
         return unconstrained
 
+    async def check_state_constraint(self, args, new_state):
+        unconstrained = True
+        if "constrain_state" in args:
+            unconstrained = utils.check_state(self.logger, new_state, args["constrain_state"])
+
+        return unconstrained
+
     #
     # Workers
     #
@@ -735,7 +742,7 @@ class Threading:
                     unconstrained = False
             if not await self.check_time_constraint(self.AD.app_management.app_config[name], name):
                 unconstrained = False
-            elif not await self.check_days_constraint(self.AD.app_management.app_config[name], name):
+            elif not await self.check_days_constraint(self.AD.app_management.app_config[name]):
                 unconstrained = False
 
         #
@@ -751,9 +758,16 @@ class Threading:
                     unconstrained = False
             if not await self.check_time_constraint(myargs["kwargs"], name):
                 unconstrained = False
-            elif not await self.check_days_constraint(myargs["kwargs"], name):
+            elif not await self.check_days_constraint(myargs["kwargs"]):
                 unconstrained = False
 
+            #
+            # Lets determine the state constraint
+            #
+            if myargs["type"] == "state":
+                state_unconstrained = await self.check_state_constraint(myargs["kwargs"], myargs["new_state"])
+                unconstrained = all((unconstrained, state_unconstrained))
+
         if unconstrained:
             #
             # It's going to happen
