@@ -17,6 +17,8 @@ import json
 import inspect
 from functools import wraps
 from appdaemon.version import __version__  # noqa: F401
+from collections.abc import Iterable
+import concurrent.futures
 
 if platform.system() != "Windows":
     import pwd
@@ -177,6 +179,33 @@ class StateAttrs(dict):
         self.__dict__ = device_dict
 
 
+class EntityStateAttrs(dict):
+    def __init__(self, dict):
+
+        self.__dict__ = AttrDict.from_nested_dict(dict)
+
+
+def check_state(logger, new_state, callback_state) -> bool:
+
+    passed = False
+
+    try:
+        if isinstance(callback_state, (str, int, float)):
+            passed = new_state == callback_state
+
+        elif isinstance(callback_state, Iterable):
+            passed = new_state in callback_state
+
+        elif callback_state.__name__ == "<lambda>":  # lambda function
+            passed = callback_state(new_state)
+
+    except Exception as e:
+        logger.warning("Could not evaluate state check due to %s", e)
+        passed = False
+
+    return passed
+
+
 def sync_wrapper(coro):
     @wraps(coro)
     def inner_sync_wrapper(self, *args, **kwargs):
@@ -268,6 +297,15 @@ def _env_var_yaml(loader, node):
     return os.environ[env_var]
 
 
+def _include_yaml(loader, node):
+    filename = node.value
+    if not os.path.isfile(filename) or filename.split(".")[-1] != "yaml":
+        raise ValueError("{} is not a valid yaml file".format(filename))
+
+    with open(filename, "r") as f:
+        return yaml.load(f, Loader=yaml.SafeLoader)
+
+
 def write_to_file(yaml_file, **kwargs):
     """Used to write the app to Yaml file"""
 
@@ -306,7 +344,7 @@ def run_coroutine_threadsafe(self, coro):
         future = asyncio.run_coroutine_threadsafe(coro, self.AD.loop)
         try:
             result = future.result(self.AD.internal_function_timeout)
-        except asyncio.TimeoutError:
+        except (asyncio.TimeoutError, concurrent.futures.TimeoutError):
             if hasattr(self, "logger"):
                 self.logger.warning(
                     "Coroutine (%s) took too long (%s seconds), cancelling the task...",
